package cn.com.qmth.scancloud.controller.admin;

import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.controller.BaseController;
import cn.com.qmth.scancloud.service.BatchService;
import cn.com.qmth.scancloud.service.ExamService;
import cn.com.qmth.scancloud.vo.checkimage.CheckImageSubmitVo;
import cn.com.qmth.scancloud.vo.checkimage.RatioVo;
import cn.com.qmth.scancloud.vo.task.TaskStatusVo;
import cn.com.qmth.scancloud.vo.verify.VerifyTaskVo;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@Api(tags = "图片审核接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/check/image")
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class CheckImageController extends BaseController {

    @Autowired
    private ExamService examService;

    @Autowired
    private BatchService batchService;

    @ApiOperation(value = "修改答题卡扫描图片抽查比例")
    @PostMapping("ratio")
    public RatioVo ratio(@RequestParam Long examId, @RequestParam Double ratio) {
        return examService.updateRatio(examId, ratio);
    }

    @ApiOperation(value = "答题卡扫描图片检查任务状态")
    @PostMapping("status")
    public TaskStatusVo status(@RequestParam Long examId) {
        User user = getAccessUser();
        return batchService.getCheckImageStatus(examId, user);
    }

    @ApiOperation(value = "获取答题卡扫描图片检查任务")
    @RequestMapping(value = "get", method = RequestMethod.POST)
    public VerifyTaskVo get(@RequestParam Long examId) {
        User user = getAccessUser();
        return batchService.getCheckImageTask(examId, user);
    }

    @ApiOperation(value = "历史答题卡扫描图片检查任务")
    @RequestMapping(value = "history", method = RequestMethod.POST)
    public VerifyTaskVo history(@RequestParam Long examId, @RequestParam(required = false) Long batchId) {
        User user = getAccessUser();
        return batchService.getHistoryCheckImageTask(examId, batchId, user);
    }

    @ApiOperation(value = "答题卡扫描图片检查任务提交")
    @RequestMapping(value = "submit", method = RequestMethod.POST)
    public CheckImageSubmitVo submit(@RequestParam Long examId, @RequestParam Long batchId) {
        User user = getAccessUser();
        return batchService.submitCheckImageTask(examId, batchId, user);
    }

    @ApiOperation(value = "答题卡扫描图片检查任务释放")
    @PostMapping("release")
    public Object release(@RequestParam Long examId) {
        User user = getAccessUser();
        batchService.releaseCheckImageTask(examId, user);
        return success(true);
    }

}