package cn.com.qmth.scancloud.service.impl;

import cn.com.qmth.scancloud.bean.ExamConfigDomain;
import cn.com.qmth.scancloud.bean.ImportExamDomain;
import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.client.MarkingcloudApiClient;
import cn.com.qmth.scancloud.dao.ExamDao;
import cn.com.qmth.scancloud.entity.*;
import cn.com.qmth.scancloud.enums.*;
import cn.com.qmth.scancloud.exception.ParameterExceptions;
import cn.com.qmth.scancloud.model.ExamInfoDto;
import cn.com.qmth.scancloud.model.StudentInfo;
import cn.com.qmth.scancloud.service.*;
import cn.com.qmth.scancloud.util.PageUtil;
import cn.com.qmth.scancloud.vo.*;
import cn.com.qmth.scancloud.vo.checkimage.RatioVo;
import cn.com.qmth.scancloud.vo.examinfo.ExamConfig;
import cn.com.qmth.scancloud.vo.examinfo.ExamInfoVo;
import cn.com.qmth.scancloud.vo.examinfo.ExamQuery;
import cn.com.qmth.scancloud.vo.scanexaminfo.ScanExamInfoVo;
import cn.com.qmth.scancloud.vo.scanexamlist.ScanExamListQuery;
import cn.com.qmth.scancloud.vo.scanexamlist.ScanExamListVo;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.retrofit.exception.RetrofitResponseError;
import com.qmth.boot.core.retrofit.utils.SignatureInfo;
import com.qmth.boot.core.security.exception.AuthorizationException;
import com.qmth.boot.mybatis.query.BaseQuery;
import com.qmth.boot.tools.signature.SignatureType;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class ExamServiceImpl extends ServiceImpl<ExamDao, ExamEntity> implements ExamService {

    @Autowired
    private SessionService sessionService;

    @Autowired
    private AnswerCardService answerCardService;

    @Autowired
    private PackageResultService packageResultService;

    @Autowired
    private PackageCardService packageCardService;

    @Autowired
    private OmrTaskService omrTaskService;

    @Autowired
    private PackageTaskService packageTaskService;

    @Autowired
    private StudentService studentService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private SystemConfigService systemConfigService;

    @Autowired
    private PaperService paperService;

    @Autowired
    private BatchService batchService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private MarkingcloudApiClient apiClient;

    @Autowired
    private OmrGroupService omrGroupService;

    @Autowired
    private UserService userService;

    @Autowired
    private QuestionService questionService;

    @Override
    public PageResult<ExamVo> pageQuery(ExamQuery query, User user) {
        if (query.getEnable() == null) {
            throw new ParameterException("是否启用不能为空");
        }
        IPage<ExamVo> iPage = this.baseMapper
                .pageExam(new Page<>(query.getPageNumber(), query.getPageSize()), query, user.getSchoolId());
        return PageUtil.of(iPage);
    }

    @Override
    public ExamInfoVo getExamInfoVo(Long examId, User accessUser) {
        ExamEntity exam = this.getById(examId);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(accessUser.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        ScanAnswerInfoVo info = studentService.scanAnswerInfo(examId);
        ExamInfoVo vo = new ExamInfoVo();
        vo.init();
        vo.setId(examId);
        vo.setName(exam.getName());
        vo.setSchoolName(exam.getSchoolName());
        vo.getExamConfig().setAllowUnexistPaper(exam.getAllowUnexistPaper());
        vo.getExamConfig().setAnswerFrontCardType(exam.getAnswerFrontCardType());
        vo.getExamConfig().setImageTransferMode(exam.getImageTransferMode());
        vo.getExamConfig().setEnableSinglePageAnswer(exam.getEnableSinglePageAnswer());
        vo.getExamConfig().setMode(exam.getMode());
        vo.getExamConfig().setScanByPackage(exam.getScanByPackage());
        vo.getExamConfig().setEnableSyncVerify(exam.getEnableSyncVerify());
        vo.getExamConfig().setPaperTypeBarcodeContent(exam.getPaperTypeBarcodeContent());
        vo.getExamConfig().setAbsentBarcodeContent(exam.getAbsentBarcodeContent());

        vo.setSubjectConfig(subjectService.listConfigByExamId(examId));
        vo.getUpload().setEnable(exam.getEnableUpload());
        vo.getUpload().setProgress(studentService.getUploadProgress(examId));

        vo.getExamData().setAnswerCard(answerCardService.getCountByExam(examId));
        vo.getExamData().setPackageCard(packageCardService.getCountByExam(examId));
        vo.getExamData().setProgress(0d);
        vo.getExamData().setStudentCount(info.getTotalCount());
        vo.getExamData().setSubjectCount(subjectService.countByExamId(examId));
        vo.getExamData().setSynching(concurrentService.isLocked(LockType.EXAM_SYNC + "-" + examId));
        vo.getExamData().setUpdateTime(exam.getDataSyncTime());

        SystemConfigEntity config = systemConfigService.findConfig();
        boolean enableLogin = true;
        if (config != null && config.getScannerEnableLogin() != null) {
            enableLogin = config.getScannerEnableLogin();
        }
        vo.getScanner().setEnableLogin(enableLogin);
        vo.getScanner().setOnlineCount(sessionService.getOnlineScannerCount());
        vo.getScanner().setPassword(config == null ? null : config.getScannerPassword());

        vo.getAnswerScan().setAbsentSuspectCount(info.getAbsentSuspectCount());
        vo.getAnswerScan().setAssignedCount(info.getAssignedCount());
        vo.getAnswerScan().setOmrAbsentCount(info.getOmrAbsentCount());
        vo.getAnswerScan().setBatchCount(batchService.getValidCountByExam(examId));
        vo.getAnswerScan().setIncompleteCount(info.getIncompleteCount());
        vo.getAnswerScan().setManualAbsentCount(info.getManualAbsentCount());
        vo.getAnswerScan().setScannedCount(info.getScannedCount());
        vo.getAnswerScan().setTotalCount(info.getTotalCount());
        vo.getAnswerScan().setUnexistCount(info.getUnexistCount());
        vo.getAnswerScan().setMismatchCount(paperService.getMismatchCountByExam(examId));

        vo.getPackageScan().setScannedCount(packageResultService.getCountByExam(examId));
        vo.getPackageScan().setTotalCount(studentService.getPackageCountByExam(examId));

        vo.getVerifyTask().setTodoCount(batchService.getVerifyCount(examId));

        vo.getOmrTask().setTotalCount(omrTaskService.getCountByExam(examId));
        vo.getOmrTask().setTodoCount(omrTaskService.getCountByExamAndStatus(examId, TaskStatus.WAITING));
        vo.getOmrTask().setUnarbitrateCount(omrTaskService.getCountByExamAndStatus(examId, TaskStatus.WAIT_ARBITRATE));

        vo.getPackageTask().setTodoCount(
                packageTaskService.getCountByExamAndStatus(examId, TaskStatus.WAITING, TaskStatus.WAIT_ARBITRATE));
        vo.getPackageTask().setTotalCount(packageTaskService.getCountByExam(examId));

        vo.getImageCheckTask().setCheckRatio(exam.getImageCheckRatio());
        vo.getImageCheckTask()
                .setFinishCount(batchService.getCheckCountByExamId(examId, CheckStatus.FINISH, accessUser));
        vo.getImageCheckTask()
                .setTodoCount(batchService.getCheckCountByExamId(examId, CheckStatus.WAITING, accessUser));

        vo.getAuditor().setOnlineCount(sessionService.getOnlineAuditorCount());
        vo.getAuditor().setUserCount(userService.countByRole(accessUser.getSchoolId(),Role.AUDITOR));
        return vo;
    }

    @Override
    public ScanExamInfoVo getScanExamInfo(Long id, User accessUser) {
        ExamEntity exam = this.getById(id);
        if (exam == null || (exam.getEnable() != null && !exam.getEnable())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        ScanExamInfoVo vo = new ScanExamInfoVo();
        vo.init();
        vo.setId(id);
        vo.setName(exam.getName());
        vo.setSchoolName(exam.getSchoolName());
        vo.getExamConfig().setAllowUnexistPaper(exam.getAllowUnexistPaper());
        vo.getExamConfig().setAnswerFrontCardType(exam.getAnswerFrontCardType());
        vo.getExamConfig().setImageTransferMode(exam.getImageTransferMode());
        vo.getExamConfig().setEnableSinglePageAnswer(exam.getEnableSinglePageAnswer());
        vo.getExamConfig().setMode(exam.getMode());
        vo.getExamConfig().setScanByPackage(exam.getScanByPackage());
        vo.getExamConfig().setEnableSyncVerify(exam.getEnableSyncVerify());
        vo.getExamConfig().setAbsentBarcodeContent(exam.getAbsentBarcodeContent());
        vo.getExamConfig().setPaperTypeBarcodeContent(exam.getPaperTypeBarcodeContent());

        vo.getAnswerScan().setCardCount(answerCardService.getCountByExam(id));
        vo.getAnswerScan().setTotalCount(studentService.getCountByExam(id));
        vo.getAnswerScan().setScannedCount(studentService.getCountByExamAndScanStatus(id, ScanStatus.SCANNED));

        vo.getPackageScan().setCardCount(packageCardService.getCountByExam(id));
        vo.getPackageScan().setTotalCount(studentService.getPackageCountByExam(id));
        vo.getPackageScan().setScannedCount(packageResultService.getCountByExam(id));

        vo.getOmrTask().setTodoCount(omrTaskService.getCountByExamAndStatus(id, TaskStatus.WAITING));
        vo.getOmrTask().setFinishCount(omrTaskService.getCountByExamAndStatus(id, TaskStatus.PROCESSED));

        vo.getPackageTask().setTodoCount(packageTaskService.getCountByExamAndStatus(id, TaskStatus.WAITING));
        vo.getPackageTask().setFinishCount(packageTaskService.getCountByExamAndStatus(id, TaskStatus.PROCESSED));

        vo.setSubjectConfig(subjectService.listConfigByExamId(id));
        return vo;
    }

    @Override
    public List<ScanExamListVo> getScanExamList() {
        ScanExamListQuery query = new ScanExamListQuery();
        query.setEnable(true);
        return this.baseMapper.getExamList(query);
    }

    @Transactional
    @Override
    public PageResult<ExamRemoteVo> listRemote(BaseQuery<?> query, User user) {
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (SystemMode.STANDALONE.equals(conf.getSystemMode())) {
            return PageUtil.emptyPage();
        }
        SignatureInfo signatureInfo = new SignatureInfo(SignatureType.TOKEN, user.getAccount(),
                user.getMarkingCloudToken());
        int pageNumber = 1;
        int pageSize = 20;
        List<ExamInfoDto> yunList = new ArrayList<>();
        try {
            List<ExamInfoDto> list = apiClient.examQuery(signatureInfo.toString(), pageNumber, pageSize);
            yunList.addAll(list);
            while (list.size() != 0) {
                pageNumber++;
                list = apiClient.examQuery(signatureInfo.toString(), pageNumber, pageSize);
                yunList.addAll(list);
            }
        } catch (RetrofitResponseError e) {
            if (e.getCode() == 401) {
                sessionService.removeSchoolSession(user.getSchoolId());
            }
            log.error("list remote exam error, account=" + user.getAccount() + ", schoolId=" + user.getSchoolId()
                    + ", token=" + user.getMarkingCloudToken(), e);
            throw e;
        } catch (Exception e) {
            log.error("list remote exam error, account=" + user.getAccount() + ", schoolId=" + user.getSchoolId()
                    + ", token=" + user.getMarkingCloudToken(), e);
            throw new StatusException("获取未开启的考试出错", e);
        }
        List<ExamInfoDto> remoteList = new ArrayList<ExamInfoDto>();
        for (ExamInfoDto infoDto : yunList) {
            if (this.getById(infoDto.getId()) == null) {
                remoteList.add(infoDto);
            }
        }
        List<ExamRemoteVo> vos = new ArrayList<ExamRemoteVo>();
        for (ExamInfoDto examInfoDto : remoteList) {
            ExamRemoteVo vo = new ExamRemoteVo();
            vo.setExamTime(examInfoDto.getExamTime());
            vo.setId(examInfoDto.getId());
            vo.setName(examInfoDto.getName());
            vo.setScanConfig(examInfoDto.getConfig());
            vos.add(vo);
        }
        List<ExamRemoteVo> page = (List<ExamRemoteVo>) PageUtil
                .startPage(vos, (int) query.getPageNumber(), (int) query.getPageSize());
        PageResult<ExamRemoteVo> result = new PageResult<>();
        result.setPageNumber(query.getPageNumber());
        result.setPageSize(query.getPageSize());
        result.setResult(page);
        result.setTotalCount(vos.size());
        return result;
    }

    @Transactional
    @Override
    public ExamConfigVo config(User user, ExamConfigDomain domain) {
        if (!Role.SCHOOL_ADMIN.equals(user.getRole())) {
            throw AuthorizationException.NO_PERMISSION;
        }
        ExamEntity exam = this.getById(domain.getId());
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (SystemMode.STANDALONE.equals(conf.getSystemMode())) {
            if (domain.getAllowUnexistPaper() != null) {
                exam.setAllowUnexistPaper(domain.getAllowUnexistPaper());
            }
            if (domain.getEnableSinglePageAnswer() != null) {
                exam.setEnableSinglePageAnswer(domain.getEnableSinglePageAnswer());
            }
            if (domain.getEnableSyncVerify() != null) {
                exam.setEnableSyncVerify(domain.getEnableSyncVerify());
            }
            if (domain.getScanByPackage() != null) {
                exam.setScanByPackage(domain.getScanByPackage());
            }
            if (domain.getAbsentBarcodeContent() != null) {
                exam.setAbsentBarcodeContent(domain.getAbsentBarcodeContent());
            }
            if (domain.getAnswerFrontCardType() != null) {
                exam.setAnswerFrontCardType(domain.getAnswerFrontCardType());
            }
            if (domain.getImageTransferMode() != null) {
                exam.setImageTransferMode(domain.getImageTransferMode());
            }
            if (domain.getMode() != null) {
                exam.setMode(domain.getMode());
            }
            if (domain.getPaperTypeBarcodeContent() != null) {
                exam.setPaperTypeBarcodeContent(domain.getPaperTypeBarcodeContent());
            }
        } else {
            if (domain.getAllowUnexistPaper() == null) {
                domain.setAllowUnexistPaper(exam.getAllowUnexistPaper());
            }
            if (domain.getEnableSinglePageAnswer() == null) {
                domain.setEnableSinglePageAnswer(exam.getEnableSinglePageAnswer());
            }
            if (domain.getEnableSyncVerify() == null) {
                domain.setEnableSyncVerify(exam.getEnableSyncVerify());
            }
            if (domain.getScanByPackage() == null) {
                domain.setScanByPackage(exam.getScanByPackage());
            }
            if (domain.getAbsentBarcodeContent() == null) {
                domain.setAbsentBarcodeContent(exam.getAbsentBarcodeContent());
            }
            if (domain.getAnswerFrontCardType() == null) {
                domain.setAnswerFrontCardType(exam.getAnswerFrontCardType());
            }
            if (domain.getImageTransferMode() == null) {
                domain.setImageTransferMode(exam.getImageTransferMode());
            }
            if (domain.getMode() == null) {
                domain.setMode(exam.getMode());
            }
            if (domain.getPaperTypeBarcodeContent() == null) {
                domain.setPaperTypeBarcodeContent(exam.getPaperTypeBarcodeContent());
            }
            SignatureInfo signatureInfo = new SignatureInfo(SignatureType.TOKEN, user.getAccount(),
                    user.getMarkingCloudToken());
            JSONObject config = JSONObject.fromObject(domain);
            try {
                ExamInfoDto examInfo = apiClient
                        .examConfig(signatureInfo.toString(), domain.getId(), config.toString());
                exam.setId(domain.getId());
                exam.setName(examInfo.getName());
                exam.setSchoolId(user.getSchoolId());
                exam.setSchoolName(examInfo.getSchoolName());
                exam.setAbsentBarcodeContent(examInfo.getConfig().getAbsentBarcodeContent());
                exam.setAllowUnexistPaper(examInfo.getConfig().getAllowUnexistPaper());
                exam.setAnswerFrontCardType(examInfo.getConfig().getAnswerFrontCardType());
                exam.setImageTransferMode(examInfo.getConfig().getImageTransferMode());
                exam.setEnableSyncVerify(examInfo.getConfig().getEnableSyncVerify());
                exam.setEnableSinglePageAnswer(examInfo.getConfig().getEnableSinglePageAnswer());
                exam.setMode(examInfo.getConfig().getMode());
                exam.setPaperTypeBarcodeContent(examInfo.getConfig().getPaperTypeBarcodeContent());
                exam.setScanByPackage(examInfo.getConfig().getScanByPackage());
            } catch (RetrofitResponseError e) {
                if (e.getCode() == 401) {
                    sessionService.userLogout(user);
                    throw AuthorizationException.SIGNATURE_INVALID;
                } else {
                    throw e;
                }
            } catch (Exception e) {
                throw new StatusException("保存考试配置出错examId:" + exam.getId(), e);
            }
        }
        this.saveOrUpdate(exam);
        return new ExamConfigVo(exam);
    }

    @Override
    public Double syncProgress(User user, Long examId) {
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (SystemMode.STANDALONE.equals(conf.getSystemMode())) {
            return 0d;
        }
        int studentCount = studentService.getCountByExam(examId);
        SignatureInfo signatureInfo = new SignatureInfo(SignatureType.TOKEN, user.getAccount(),
                user.getMarkingCloudToken());
        try {
            long totalCount = apiClient.studentCount(signatureInfo.toString(), examId);
            return (double) (studentCount * 100 / totalCount);
        } catch (RetrofitResponseError e) {
            if (e.getCode() == 401) {
                sessionService.userLogout(user);
                throw AuthorizationException.SIGNATURE_INVALID;
            } else {
                throw e;
            }
        } catch (StatusException e) {
            throw e;
        } catch (Exception e) {
            throw new StatusException("获取同步进度出错examId:" + examId, e);
        }
    }

    @Transactional
    @Override
    public void sync(User user, Long examId) {
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (SystemMode.STANDALONE.equals(conf.getSystemMode())) {
            return;
        }
        ExamEntity exam = this.getById(examId);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        SignatureInfo signatureInfo = new SignatureInfo(SignatureType.TOKEN, user.getAccount(),
                user.getMarkingCloudToken());
        long now = System.currentTimeMillis();
        try {
            int pageNumber = 1;
            int pageSize = 100;
            List<StudentEntity> saveList = new ArrayList<StudentEntity>();
            List<SubjectEntity> subjectList = new ArrayList<SubjectEntity>();
            Map<String, SubjectEntity> subjectMap = new HashMap<String, SubjectEntity>();
            List<StudentInfo> students = apiClient.getStudents(signatureInfo.toString(), examId, pageNumber, pageSize);
            while (students.size() != 0) {
                List<Long> ids = new ArrayList<Long>();
                students.forEach((e) -> {
                    ids.add(e.getId());
                });
                List<StudentEntity> list = studentService.listByIds(ids);
                Map<Long, StudentEntity> map = new HashMap<Long, StudentEntity>();
                list.forEach((e) -> {
                    map.put(e.getId(), e);
                });
                for (StudentInfo studentInfo : students) {
                    StudentEntity student = map.get(studentInfo.getId());
                    if (student == null) {
                        student = new StudentEntity();
                        student.setId(studentInfo.getId());
                        student.setExamId(examId);
                        student.setPaperType(studentInfo.getPaperType());
                        student.setAbsentSuspect(false);
                        student.setAssigned(false);
                        student.setIncomplete(false);
                        student.setOmrAbsent(false);
                        student.setQuestionFilled(false);
                        student.setStatus(ScanStatus.UNEXIST);
                    } else {
                        //考生已存在且未扫描状态下，才同步云端现有试卷类型
                        //否则需要保留已扫描的试卷类型
                        if (student.getStatus() == ScanStatus.UNEXIST) {
                            student.setPaperType(studentInfo.getPaperType());
                        }
                    }
                    //更新必填属性
                    student.setExamNumber(studentInfo.getExamNumber());
                    student.setStudentCode(studentInfo.getStudentCode());
                    student.setName(studentInfo.getName());
                    student.setSubjectCode(studentInfo.getSubjectCode());
                    //更新可选属性
                    student.setExamRoom(studentInfo.getExamRoom() != null && studentInfo.getExamRoom().length() > 0 ?
                            studentInfo.getExamRoom() :
                            null);
                    student.setExamSite(studentInfo.getExamSite() != null && studentInfo.getExamSite().length() > 0 ?
                            studentInfo.getExamSite() :
                            null);
                    student.setPackageCode(
                            studentInfo.getPackageCode() != null && studentInfo.getPackageCode().length() > 0 ?
                                    studentInfo.getPackageCode() :
                                    null);
                    //更新时间属性
                    student.setCreatorId(user.getId());
                    student.setCreateTime(now);
                    student.setUpdaterId(user.getId());
                    student.setUpdateTime(now);
                    saveList.add(student);
                    SubjectEntity subject = subjectMap.get(studentInfo.getSubjectCode());
                    if (subject == null) {
                        subject = new SubjectEntity();
                        subject.setExamId(examId);
                        subject.setCode(studentInfo.getSubjectCode());
                        subject.setName(studentInfo.getSubjectName());
                        subject.setCreateTime(now);
                        subject.setCreatorId(user.getId());
                        subject.setUpdaterId(user.getId());
                        subject.setUpdateTime(now);
                        subjectList.add(subject);
                        subjectMap.put(studentInfo.getSubjectCode(), subject);
                    }
                }
                if (saveList.size() > 1000) {
                    studentService.saveOrUpdateBatch(saveList);
                    saveList = new ArrayList<>();
                }
                pageNumber++;
                students = apiClient.getStudents(signatureInfo.toString(), examId, pageNumber, pageSize);
            }
            for (String subjectCode : subjectMap.keySet()) {
                List<QuestionEntity> questions = apiClient.getQuestion(signatureInfo.toString(), examId, subjectCode);
                if (!questions.isEmpty()) {
                    for (QuestionEntity question : questions) {
                        question.setSubjectCode(subjectCode);
                        question.setExamId(examId);
                    }
                    questionService.deleteByExamIdAndSubjectCode(examId, subjectCode);
                    questionService.saveBatch(questions);
                }
            }
            studentService.saveOrUpdateBatch(saveList);
            subjectService.saveOrUpdateBatchByMultiId(subjectList);
            studentService.deletetByExamIdAndUnCreateTime(examId, now);
            this.updateDataSyncTime(examId);
            //一并同步卡格式
            answerCardService.sync(user, examId);
        } catch (RetrofitResponseError e) {
            if (e.getCode() == 401) {
                sessionService.userLogout(user);
                throw AuthorizationException.SIGNATURE_INVALID;
            } else {
                throw e;
            }
        }
    }

    @Transactional
    @Override
    public ExamInitVo init(User user, ExamConfigDomain domain) {
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (SystemMode.STANDALONE.equals(conf.getSystemMode())) {
            throw new ParameterException("本地模式不支持此功能");
        }
        if (ExamMode.CET.equals(domain.getMode())) {
            throw new ParameterException("云阅卷模式下不支持CET考试");
        }
        if (!Role.SCHOOL_ADMIN.equals(user.getRole())) {
            throw AuthorizationException.NO_PERMISSION;
        }
        ExamEntity exam = this.getById(domain.getId());
        if (exam != null) {
            throw new ParameterException("考试已经存在");
        }
        // if (domain.getAbsentBarcodeContent() == null) {
        // throw new ParameterException("AbsentBarcodeContent 不能为空");
        // }
        // if (domain.getPaperTypeBarcodeContent().isEmpty()) {
        // throw new ParameterException("PaperTypeBarcodeContent 不能为空");
        // }
        exam = new ExamEntity();
        exam.setEnable(true);
        //独立模式默认不开启上传
        exam.setEnableUpload(!SystemMode.STANDALONE.equals(conf.getSystemMode()));
        //图片抽查比例默认为0
        exam.setImageCheckRatio(0d);
        exam.setMode(domain.getMode());
        exam.setAllowUnexistPaper(domain.getAllowUnexistPaper());
        exam.setEnableSinglePageAnswer(domain.getEnableSinglePageAnswer());
        exam.setEnableSyncVerify(domain.getEnableSyncVerify());
        exam.setScanByPackage(domain.getScanByPackage());
        exam.setAnswerFrontCardType(domain.getAnswerFrontCardType());
        exam.setImageTransferMode(domain.getImageTransferMode());
        exam.setAbsentBarcodeContent(StringUtils.trimToEmpty(domain.getAbsentBarcodeContent()));
        exam.setPaperTypeBarcodeContent(
                domain.getPaperTypeBarcodeContent() != null ? domain.getPaperTypeBarcodeContent() : new ArrayList<>());
        SignatureInfo signatureInfo = new SignatureInfo(SignatureType.TOKEN, user.getAccount(),
                user.getMarkingCloudToken());
        JSONObject config = JSONObject.fromObject(domain);
        try {
            ExamInfoDto examInfo = apiClient.examConfig(signatureInfo.toString(), domain.getId(), config.toString());
            exam.setId(domain.getId());
            exam.setName(examInfo.getName());
            exam.setSchoolId(user.getSchoolId());
            exam.setSchoolName(examInfo.getSchoolName());
            this.save(exam);
            omrGroupService.addFixOmrCondition(user.getId(), exam.getId());
        } catch (RetrofitResponseError e) {
            if (e.getCode() == 401) {
                sessionService.userLogout(user);
                throw AuthorizationException.SIGNATURE_INVALID;
            } else {
                throw e;
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new StatusException("保存考试配置出错examId:" + exam.getId(), e);
        }
        ExamInitVo vo = new ExamInitVo();
        vo.setExamConfig(new ExamConfig(exam));
        vo.setSubjectConfig(subjectService.listConfigByExamId(exam.getId()));
        return vo;
    }

    @Transactional
    @Override
    public ExamEntity save(ImportExamDomain domain) {
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (SystemMode.MARKING_CLOUD.equals(conf.getSystemMode())) {
            throw new ParameterException("非本地模式不支持此功能");
        }
        if (domain.getAllowUnexistPaper() == null) {
            throw new ParameterException("AllowUnexistPaper 不能为空");
        }
        if (domain.getEnableSinglePageAnswer() == null) {
            throw new ParameterException("EnableSinglePageAnswer 不能为空");
        }
        if (domain.getEnableSyncVerify() == null) {
            throw new ParameterException("EnableSyncVerify 不能为空");
        }
        if (domain.getScanByPackage() == null) {
            throw new ParameterException("ScanByPackage 不能为空");
        }
        if (domain.getAbsentBarcodeContent() == null) {
            throw new ParameterException("AbsentBarcodeContent 不能为空");
        }
        if (domain.getAnswerFrontCardType() == null) {
            throw new ParameterException("AnswerFrontCardType 不能为空");
        }
        if (domain.getImageTransferMode() == null) {
            domain.setImageTransferMode(ImageTransferMode.OFF);
            // throw new ParameterException("ImageTransferMode 不能为空");
        }
        if (domain.getMode() == null) {
            throw new ParameterException("ExamMode 不能为空");
        }
        if (domain.getPaperTypeBarcodeContent() == null) {
            throw new ParameterException("PaperTypeBarcodeContent 不能为空");
        }
        ExamEntity exam = new ExamEntity();
        exam.setId(domain.getId());
        exam.setSchoolId(domain.getSchoolId());
        exam.setSchoolName(domain.getSchoolName());
        exam.setName(domain.getName());
        exam.setEnable(true);
        exam.setEnableUpload(false);
        exam.setImageCheckRatio(0d);
        exam.setAllowUnexistPaper(domain.getAllowUnexistPaper());
        exam.setEnableSinglePageAnswer(domain.getEnableSinglePageAnswer());
        exam.setEnableSyncVerify(domain.getEnableSyncVerify());
        exam.setScanByPackage(domain.getScanByPackage());
        exam.setAbsentBarcodeContent(domain.getAbsentBarcodeContent());
        exam.setAnswerFrontCardType(domain.getAnswerFrontCardType());
        exam.setImageTransferMode(domain.getImageTransferMode());
        exam.setMode(domain.getMode());
        exam.setPaperTypeBarcodeContent(domain.getPaperTypeBarcodeContent());
        if (getById(domain.getId()) == null) {
            omrGroupService.addFixOmrCondition(1L, exam.getId());
        }
        this.saveOrUpdate(exam);
        return exam;
    }

    @Override
    public RatioVo updateRatio(Long examId, Double ratio) {
        if (ratio == null || ratio < 0 || ratio > 1) {
            throw new ParameterException("ratio 不符合");
        }
        ExamEntity exam = this.getById(examId);
        if (exam == null) {
            throw new ParameterException("Exam 不能为空");
        }
        exam.setImageCheckRatio(ratio);
        this.updateById(exam);
        return new RatioVo(ratio);
    }
    
    @Transactional
    @Override
    public void updateDataSyncTime(Long examId) {
        ExamEntity exam = this.getById(examId);
        if (exam == null) {
            throw new ParameterException("Exam 不能为空");
        }
        exam.setDataSyncTime(System.currentTimeMillis());
        this.updateById(exam);
        
    }
    
    @Transactional
    @Override
    public void updateCardSyncTime(Long examId) {
        ExamEntity exam = this.getById(examId);
        if (exam == null) {
            throw new ParameterException("Exam 不能为空");
        }
        exam.setCardSyncTime(System.currentTimeMillis());
        this.updateById(exam);
        
    }

}
