package cn.com.qmth.scancloud.task;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.enums.LockType;
import cn.com.qmth.scancloud.service.AnswerCardService;

public class CardSyncThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(CardSyncThread.class);

    private User user;

    private Long examId;

    private AnswerCardService answerCardService;

    private ConcurrentService concurrentService;

    public CardSyncThread(User user, Long examId, AnswerCardService answerCardService,
            ConcurrentService concurrentService) {
        this.examId = examId;
        this.user = user;
        this.answerCardService = answerCardService;
        this.concurrentService = concurrentService;
    }

    @Override
    public void run() {
        if (examId == null) {
            return;
        }
        if (user == null) {
            return;
        }
        try {
            if( concurrentService.getLock(LockType.CARD_SYNC + "-" + examId).tryLock()) {
                answerCardService.sync(user, examId);
            }
        } catch (Exception e) {
            log.error("card sync error", e);
            throw new StatusException("卡格式同步失败");
        } finally {
            concurrentService.getLock(LockType.CARD_SYNC + "-" + examId).unlock();
        }
    }

}
